#!/usr/bin/python
'''
hpc_example.py -t <wait time> -l <list length>
'''

from getopt import getopt, GetoptError
from sys import argv, exit
from psutil import Process
from os import getpid
from time import sleep
from platform import node

def parseCommand(wait_time=10, list_length=0):
    '''
    parse the command and set the wait time and list length accordingly
    '''
    usage = 'hpc_example.py -t <wait time> -l <list length>'
    # handle the arguments
    try:
       opts, args = getopt(argv,"ht:l:",["time=","length="])
    except GetoptError as err:
       print(err)
       print(usage)
       exit(2)
    for opt, arg in opts:
       if opt == '-h':
          print(usage)
          exit()
       elif opt in ("-t", "--time"):
          wait_time = max(wait_time, float(arg))
       elif opt in ("-l", "--length"):
          list_length = max(list_length, int(arg))
    return (wait_time, list_length)

def use_memory(list_length):
    '''
    create a list of length list_length
    and return the amount of memory being used by this process in MB
    '''
    memory_use_list = list(range(list_length))
    process = Process(getpid())
    memory_used_MB = process.memory_info().rss / 1_000_000
    return memory_used_MB

def main(argv):
   wait_time, list_length = parseCommand()
   print(f'Current host is: {node()}')
   print(f'Wait time is: {wait_time} seconds')
   print(f'List length is: {list_length}')
   mem_used = use_memory(list_length)
   print(f'Memory usage: {mem_used} MB')
   sleep(wait_time) # wait for the specified number of seconds

if __name__ == "__main__":
   main(argv[1:])
